* ============================================================================== 
* Complementary Stata code to paper "Does Unobservable Heterogeneity Matter for 
* Portfolio-Based Asset Pricing Tests?" by Daniel Hoechle, Markus Schmid, and 
* Heinz Zimmermann (2024)
* ============================================================================== 
*
* ------------------------------------------------------------------------------
* Content of this tutorial
* ------------------------------------------------------------------------------
*
* 1. Empirical validation of propositions 1 to 3 in the paper. It is shown
*    that the GPS-model is capable to reproduce the results of conventional
*    portfolio sorts by estimating a firm-level pooled WLS regression. 
*
* 2. Implementation of the "portfolio sorts specification test" and the more
*    general "GPS-model specification test" proposed in the paper. If the
*    tests reject the null hypothesis that the random effects assumption holds,
*    then a GPS-model estimated with the fixed effects (FE) estimator is 
*    preferred to an analysis relying on conventional portfolio sorts.
*
* 3. Estimation of GPS-models containing multivariate and continuous firm
*    characteristics with fixed effects estimation (as well as with less robust
*    pooled OLS/WLS).
*
*
* ------------------------------------------------------------------------------
* Notes and Instructions
* ------------------------------------------------------------------------------
*
* Our paper "Does Unobservable Heterogeneity Matter for Portfolio-Based Asset 
* Pricing Tests?" is accompanied with two user-written Stata commands:
*
* xtgps		  This command estimates all sorts of GPS-models. Furthermore, the 
*           program also performs the GPS-model specification test. For details,
*           type "help xtgps".
*
* xtpsort   This command assists in performing a conventional portfolio sorts
*           analysis. First, the program computes for each period the (weighted) 
*           portfolio (excess) return. Second, it then evaluates the portfolio's
*           (risk-adjusted) performance by estimating a  K-factor time-series 
*           regression model with the portfolio's (excess) return as the 
*           dependent variable. The program can also compare the performance 
*           of two portfolios. For details, type "help xtpsort".
*
* Please store do-file "GPS-model, StataTutorial.do" and the sample data (file
* "HSZ_DemoData.dta") in a dedicated folder and adjust row 95 in this do-file 
* with a reference to the respective folder.
*
* The tutorial makes use of Ben Jann's -estout- package. To install the -estout-
* package in Stata, please type:
* . net install estout, from("http://fmwww.bc.edu/RePEc/bocode/e")
*
*
* ------------------------------------------------------------------------------
* Description of dataset "HSZ_DemoData.dta"
* ------------------------------------------------------------------------------
*
* Content
* -------
* - Subset comprising 2,000 randomly drawn permnos from the prepared CRSP-
*   Compustat merged database. The dataset is anonymized in the sense that 
*   actual permno's are overwritten with a new, randomly generated "permno".
* - Each permno contains at least one observation with non-missing information
*   on operating profitability (variable OA).
* - The sample period goes from December 1990 through December 2010.
*
*
* Variable description
* --------------------
* - permno  (anonymized) permno
* - mdate   Month in Stata's time-series format
* - XRet    Monthly stock return in excess of the risk-free return
* - OA 		  Operating profitability scaled by total assets
* - OA10    Operating profitability decile (based on NYSE-breakpoints) of firm i
*           (Portfolios are formed by end of June and then held for the next year)
* - OA10w   Month t value weight of firm i in decile portfolio OA10
*           (weight is based on me_lag; weights in each portfolio sum to 1)
* - GA		  Gross profitability scaled by total assets
* - me_lag  Lagged Market Equity (CRSP based)
* - RMRF    Monthly return of the market factor (from Ken French's data library)
* - SMB     Monthly return of the size factor (from Ken French's data library)
* - HML     Monthly return of the value factor (from Ken French's data library)
*
* ==============================================================================  
* Daniel Hoechle, 26. August 2024 (First version: 06. November 2017)
* ==============================================================================  


  version 18.5
  cls
  set more off, permanently
  clear all
	
* Directory where the sample data "HSZ_DemoData.dta" is stored (please adjust)
	global DataDir = c(pwd)

	
							
* ==============================================================================  
* Part A - Evaluating the performance of a single portfolio with the GPS-model
*          ==> Empirical validation of Proposition 1 in the paper
* ==============================================================================  

* Load the dataset
  cls  
	eststo clear
  use "$DataDir\HSZ_DemoData.dta", clear

	
* (1) Prtfolio sorts approach (TS regression with Newey-West standard errors)
	eststo PSort: xtpsort XRet RMRF SMB HML if OA10==10, lag(3) ipw(OA10w) ase
	
	
* (2) GPS-model (pooled WLS estimation; Driscoll-Kraay standard errors)
  eststo GPS_WLS: xtgps XRet [aweight=OA10w] if OA10==10, tsv(RMRF SMB HML) lag(3) ase
		
	
* (3) Compare the results
  esttab * , mtitles r2 scalars(N_g) obslast star(* 0.10 ** 0.05  *** 0.01)
	
	
* Note: Driscoll and Kraay (1998) standard errors are robust to
*       heteroscedasticity, autocorrelation, and cross-sectional dependence.
	
	
	
* ==============================================================================  
* Part B - Using the GPS-model to obtain portfolio sorts (one-way sort)
*          ==> Empirical validation of Proposition 2 in the paper
* ==============================================================================  

* Load the dataset
  cls  
	eststo clear
  use "$DataDir\HSZ_DemoData.dta", clear

	
* (1) Conventional portfolio sorts
	qui forvalues q = 1(1)10 { 
		eststo PSort_Q`q': xtpsort XRet RMRF SMB HML if OA10==`q', lag(3) ipw(OA10w) ase
	}

	
* (2) GPS-model (pooled WLS estimation; Driscoll-Kraay standard errors)
  tab OA10, gen(PFg)    // Generation of portfolio dummy variables
  eststo GPS: xtgps XRet PFg* [aweight=OA10w], tsv(RMRF SMB HML) lag(3) noc ase
	
	
* (3) Compare the results

  * Results from the portfolio sorts approach
  local title "Empirical validation of Proposition 2 - Panel A: Portfolio Sort"
  esttab PSort_Q* , mtitles r2 obslast star(* 0.10 ** 0.05  *** 0.01) title("`title'")
	
	* Results from estimating a GPS-model with pooled WLS
  local title "Empirical validation of Proposition 2 - Panel B: GPS-model"
  esttab GPS , mtitles r2 obslast star(* 0.10 ** 0.05  *** 0.01) unstack title("`title'")
	
	
	
* ==============================================================================  
* Part C - Performance difference between top and bottom decile portfolio
*          ==> Empirical validation of Proposition 3 in the paper
* ==============================================================================  

* Load the dataset
  cls    
	eststo clear
  use "$DataDir\HSZ_DemoData.dta", clear
	
* Dummy variable with value one for stocks in top quintile (OA5==5) portfolio
	gen Q10 = (OA10==10)
  label var Q10 "Dummy. 1 = Obs. belongs to top decile (OA10==10) portfolio"

	
* (1) Conventional portfolio sorts

  * Analysis of the performance of the top versus bottom quintile portfolio
		eststo PS_HiLow: xtpsort XRet RMRF SMB HML if inlist(OA10, 1, 10),      ///
																group(Q10) lag(3) ipw(OA10w) ase

  * Performance of the bottom quintile portfolio
    eststo PSort_Low: xtpsort XRet RMRF SMB HML if OA10==1, lag(3) ipw(OA10w) ase
		

* (2) GPS-model estimated with pooled WLS (Driscoll-Kraay standard errors)
	eststo GPS_DK98: xtgps XRet Q10 [aweight=OA10w] if inlist(OA10, 1, 10),     ///
														  tsv(RMRF SMB HML) lag(3) ase
															

* (3) Compare the results
  esttab * , mtitles r2 obslast star(* 0.10 ** 0.05  *** 0.01) unstack       ///
	           title("Empirical validation of Proposition 3")
						 

		
* ==============================================================================  
* Part D - Portfolio Sorts Specification test
*          ==> See Section 2.3.4 "Testing the validity of standard portfolio 
*              sorts" for details
* ==============================================================================  

* Load the dataset
  cls   
  use "$DataDir\HSZ_DemoData.dta", clear
	
* Generate portfolio dummy variables
  tab OA10, gen(PFg)
	
* Perform the portfolio sorts specificaiton test	
	local DummyList ""
	forvalues pf = 2/10 {
	    local DummyList "`DummyList' PFg`pf'"
	}	
	xtgps XRet `DummyList' [aweight=OA10w], tsv(RMRF SMB HML) lag(3) ase spec notable
	
	
	
* ==============================================================================  
* Part E - Applications of the GPS-model beyond conventional portfolio sorts 
* ==============================================================================  

* Load the dataset
  cls  
	eststo clear
  use "$DataDir\HSZ_DemoData.dta", clear

* Data management: computation of time t value weights
  sort mdate permno
	by mdate: egen totme = total(me_lag)
  gen ObsWeight = me_lag/totme
	label var ObsWeight "Time t value weight of stock i"
	drop if ObsWeight==0	
	xtset		

	
* ------------------------------------------------------------------------------	
* E1) GPS-models with continuous firm characteristics (pooled WLS estimation)
* ------------------------------------------------------------------------------

* Note: For brevity, we only display the results for the decomposition of 
*       the alpha (i.e. for the coefficients in vector z_it). This is 
*       accomplished by calling -xtgps- with option "alphaonly"

* Estimation of the GPS-models
	eststo clear
	local GPSoptions "tsv(RMRF SMB HML) lag(3) ase alphaonly"
	eststo M1: qui xtgps XRet OA [aweight=ObsWeight], `GPSoptions' 
	eststo M2: qui xtgps XRet GA [aweight=ObsWeight], `GPSoptions' 
	eststo M3: qui xtgps XRet OA GA [aweight=ObsWeight], `GPSoptions' 
	
* Tabulation of the results
	local title "GPS-models: pooled WLS estimation (results for alpha decomposition only)"
	esttab *, mtitles r2 obslast star(* 0.10 ** 0.05  *** 0.01) unstack title("`title'") 

	
* ------------------------------------------------------------------------------	
* E2) GPS-model specification test
* ------------------------------------------------------------------------------
		
* Estimate the extended GPS-model (see equation (18) for details)
	eststo clear
	local GPSoptions "tsv(RMRF SMB HML) lag(3) ase spec"
	eststo M1: qui xtgps XRet OA [aweight=ObsWeight], `GPSoptions'
	eststo M2: qui xtgps XRet GA [aweight=ObsWeight], `GPSoptions' 
	eststo M3: qui xtgps XRet OA GA [aweight=ObsWeight], `GPSoptions' 
	
* Tabulation of the results (for brevity, only results relevant for the 
* GPS-model specification test are displayed)
	local title "GPS-model specification test"
	esttab *, mtitles r2 obslast star(* 0.10 ** 0.05  *** 0.01) varwidth(25)   ///
	          scalars(SpecTest_F SpecTest_pval) title("`title'") keep(Controls:)
	
	
* ------------------------------------------------------------------------------	
* E3) GPS-models with continuous firm characteristics (fixed effects estimation)
* ------------------------------------------------------------------------------
	
  * Estimation of the GPS-models (only display results for alpha decomposition)
	eststo clear
	local GPSoptions "tsv(RMRF SMB HML) lag(3) ase alphaonly fe"
	eststo M1: qui xtgps XRet OA [aweight=ObsWeight], `GPSoptions' 
	eststo M2: qui xtgps XRet GA [aweight=ObsWeight], `GPSoptions'
	eststo M3: qui xtgps XRet OA GA [aweight=ObsWeight], `GPSoptions'
	
  * Tabulation of the results
	local title "GPS-models: FE estimation (results for alpha decomposition only)"
	esttab *, mtitles obslast scalars(r2_w N_g) star(* 0.10 ** 0.05  *** 0.01) title("`title'") 
	
	
* ------------------------------------------------------------------------------	
* E4) Cross-section versus time-series predictability
* ------------------------------------------------------------------------------
	
* Data management: XS and TS components of firm characteritics
	by permno: egen double TotWeight = total(ObsWeight)
	foreach chr of varlist OA GA {
		by permno: egen double `chr'xs = total(`chr'*ObsWeight/TotWeight) if !mi(`chr')
		label var `chr'xs "XS predictability (`chr' bar i)"
		sum `chr' if !mi(`chr') [aweight=ObsWeight], meanonly
		scalar TotMean = r(mean)
		gen `chr'ts = `chr' - `chr'xs + TotMean
		label var `chr'ts "TS predictability (`chr' tilde it)"
	}

* XS vs. TS predictability of Fama-French three-factor model alphas
  eststo clear
	qui foreach chr of varlist OA GA {
	
		* Estimation of GPS-model (only display results for alpha decomposition)
			eststo `chr': xtgps XRet `chr'ts `chr'xs [aweight=ObsWeight],          ///
													tsv(RMRF SMB HML) lag(3) ase alphaonly
															 
		* Wald test on TS predictability == XS predictability
			test _b[`chr'ts]=_b[`chr'xs] 
			estadd scalar TSXS_F = r(F)
			estadd scalar TSXS_pval = r(p)	
			
	}
	
* Tabulation of the results
	local title "TS vs XS predictability of Fama-French three-factor model alpha"
  esttab *, mtitles r2(%7.6f) scalars(TSXS_F TSXS_pval) label                ///
	          star(* 0.10 ** 0.05  *** 0.01) title(`title') varwidth(35)
		
		
* End of do-file
